@@ -202,7 +202,7 @@ static gboolean do_spawn_with_pipes  (gboolean              dont_wait,
 				      gboolean		    file_and_argv_zero,
                                       GSpawnChildSetupFunc  child_setup,
                                       gpointer              user_data,
-                                      gint                 *child_handle,
+                                      GPid                 *child_handle,
                                       gint                 *standard_input,
                                       gint                 *standard_output,
                                       gint                 *standard_error,
@@ -225,7 +225,7 @@ g_spawn_async (const gchar          *working_directory,
                GSpawnFlags           flags,
                GSpawnChildSetupFunc  child_setup,
                gpointer              user_data,
-               gint                 *child_handle,
+               GPid                 *child_handle,
                GError              **error)
 {
   g_return_val_if_fail (argv != NULL, FALSE);
@@ -316,7 +316,7 @@ g_spawn_sync (const gchar          *working_directory,
 {
   gint outpipe = -1;
   gint errpipe = -1;
-  gint pid;
+  GPid pid;
   GIOChannel *outchannel = NULL;
   GIOChannel *errchannel = NULL;
   GPollFD outfd, errfd;
@@ -492,6 +492,8 @@ g_spawn_sync (const gchar          *working_directory,
   if (errpipe >= 0)
     close_and_invalidate (&errpipe);
   
+  g_spawn_close_pid(pid);
+
   if (failed)
     {
       if (outstr)
@@ -523,7 +525,7 @@ g_spawn_async_with_pipes (const gchar          *working_directory,
                           GSpawnFlags           flags,
                           GSpawnChildSetupFunc  child_setup,
                           gpointer              user_data,
-                          gint                 *child_handle,
+                          GPid                 *child_handle,
                           gint                 *standard_input,
                           gint                 *standard_output,
                           gint                 *standard_error,
@@ -644,14 +646,17 @@ do_spawn (gboolean              dont_wait,
 	  GSpawnChildSetupFunc  child_setup,
 	  gpointer              user_data)
 {
+  gchar **protected_argv;
   gchar **new_argv;
   gchar args[ARG_COUNT][10];
   gint i;
   int rc;
-  int argc = 0;
+  int argc;
 
   SETUP_DEBUG();
 
+  argc = protect_argv (argv, &protected_argv);
+
   if (stdin_fd == -1 && stdout_fd == -1 && stderr_fd == -1 &&
       (working_directory == NULL || !*working_directory) &&
       !close_descriptors &&
@@ -665,10 +670,14 @@ do_spawn (gboolean              dont_wait,
 	g_print ("doing without gspawn-win32-helper\n");
 
       if (search_path)
-	rc = spawnvp (mode, argv[0], argv);
+	rc = spawnvp (mode, argv[0], protected_argv);
       else
-	rc = spawnv (mode, argv[0], argv);
+	rc = spawnv (mode, argv[0], protected_argv);
 
+      for (i = 0; i < argc; i++)
+	g_free (protected_argv[i]);
+      g_free (protected_argv);
+      
       if (rc == -1)
 	{
 	  return DO_SPAWN_ERROR_NO_HELPER;
@@ -680,9 +689,6 @@ do_spawn (gboolean              dont_wait,
 	}
     }
 
-  while (argv[argc])
-    ++argc;
-
   new_argv = g_new (gchar *, argc + 1 + ARG_COUNT);
 
   new_argv[0] = "gspawn-win32-helper";
@@ -755,7 +761,7 @@ do_spawn (gboolean              dont_wait,
     new_argv[ARG_WAIT] = "w";
 
   for (i = 0; i <= argc; i++)
-    new_argv[ARG_PROGRAM + i] = argv[i];
+    new_argv[ARG_PROGRAM + i] = protected_argv[i];
 
   /* Call user function just before we execute the helper program,
    * which executes the program. Dunno what's the usefulness of this.
@@ -794,6 +800,10 @@ do_spawn (gboolean              dont_wait,
   if (stderr_fd >= 0)
     close (stderr_fd);
 
+  for (i = 0; i < argc; i++)
+    g_free (protected_argv[i]);
+  g_free (protected_argv);
+
   g_free (new_argv[ARG_WORKING_DIRECTORY]);
   g_free (new_argv);
 
@@ -861,7 +871,7 @@ do_spawn_with_pipes (gboolean              dont_wait,
 		     gboolean		   file_and_argv_zero,
 		     GSpawnChildSetupFunc  child_setup,
 		     gpointer              user_data,
-		     gint                 *child_handle,
+		     GPid                 *child_handle,
 		     gint                 *standard_input,
 		     gint                 *standard_output,
 		     gint                 *standard_error,
@@ -875,9 +885,6 @@ do_spawn_with_pipes (gboolean              dont_wait,
   gint helper = -1;
   gint buf[2];
   gint n_ints = 0;
-  gint i;
-  gint argc;
-  gchar **new_argv;
   
   if (!make_pipe (child_err_report_pipe, error))
     return FALSE;
@@ -891,15 +898,13 @@ do_spawn_with_pipes (gboolean              dont_wait,
   if (standard_error && !make_pipe (stderr_pipe, error))
     goto cleanup_and_fail;
 
-  argc = protect_argv (argv, &new_argv);
-
   helper = do_spawn (dont_wait,
 		     child_err_report_pipe[1],
 		     stdin_pipe[0],
 		     stdout_pipe[1],
 		     stderr_pipe[1],
 		     working_directory,
-		     new_argv,
+		     argv,
 		     envp,
 		     close_descriptors,
 		     search_path,
@@ -910,10 +915,6 @@ do_spawn_with_pipes (gboolean              dont_wait,
 		     child_setup,
 		     user_data);
       
-  for (i = 0; i < argc; i++)
-    g_free (new_argv[i]);
-  g_free (new_argv);
-
   /* Check if gspawn-win32-helper couldn't be run */
   if (helper == DO_SPAWN_ERROR_HELPER)
     {
@@ -1037,5 +1038,10 @@ make_pipe (gint     p[2],
   else
     return TRUE;
 }
-
 #endif /* !GSPAWN_HELPER */
+
+void
+g_spawn_close_pid (GPid pid)
+{
+    CloseHandle (pid);
+}
